package com.ningdatech.pmapi.projectlib.handle;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

import com.ningdatech.basic.util.CollUtils;
import com.ningdatech.basic.util.NdDateUtils;
import com.ningdatech.pmapi.projectlib.model.entity.Project;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.pmapi.common.constant.CommonConstant;
import com.ningdatech.pmapi.common.statemachine.event.ProjectStatusChangeEvent;
import com.ningdatech.pmapi.projectlib.enumeration.ProjectStatusEnum;
import com.ningdatech.pmapi.projectlib.enumeration.StepStatusEnum;
import com.ningdatech.pmapi.projectlib.manage.AnnualPlanLibManage;
import com.ningdatech.pmapi.projectlib.model.entity.NdProjectStatusChange;
import com.ningdatech.pmapi.projectlib.model.req.ProjectListReq;
import com.ningdatech.pmapi.projectlib.model.vo.AnnualPlanListItemVO;
import com.ningdatech.pmapi.projectlib.model.vo.ProcessDetailVO;
import com.ningdatech.pmapi.projectlib.service.INdProjectStatusChangeService;
import com.ningdatech.pmapi.projectlib.service.IProjectService;

import static com.ningdatech.pmapi.projectlib.enumeration.ProjectStatusEnum.*;

/**
 * 立项批复处理
 *
 * @author CMM
 * @since 2023/02/24 14:35
 */
@Order(7)
@Component
public class ProjectApprovalHandle extends AbstractProcessBusinessHandle {

    private final AnnualPlanLibManage annualPlanLibManage;
    private final INdProjectStatusChangeService projectStatusChangeService;
    private final IProjectService projectService;

    /**
     * 项目状态为：待立项批复及之后的状态
     */
    private static final List<ProjectStatusEnum> ANNUAL_PLAN_LIST_STATUS = Arrays.asList(
            TO_BE_APPROVED,
            PROJECT_APPROVED,
            TO_BE_PURCHASED,
            UNDER_CONSTRUCTION,
            TO_BE_FINALLY_INSPECTED,
            FINAL_ACCEPTANCE_IS_UNDER_REVIEW,
            FINAL_ACCEPTANCE_REVIEW_FAILED,
            ARCHIVED
    );

    public ProjectApprovalHandle(AnnualPlanLibManage annualPlanLibManage, INdProjectStatusChangeService projectStatusChangeService, IProjectService projectService){
        this.annualPlanLibManage = annualPlanLibManage;
        this.projectStatusChangeService = projectStatusChangeService;
        this.projectService = projectService;
    }

    @Override
    void businessHandle(Long projectId, List<ProcessDetailVO> processSchedule) {
        ProcessDetailVO processDetailVO = new ProcessDetailVO();
        // 根据年度计划库列表是否包含该项目以及建设方案评审流程是否通过来判断立项批复是否开始
        ProjectListReq req = new ProjectListReq();
        PageVo<AnnualPlanListItemVO> planLibList = annualPlanLibManage.annulPlanLibList(req);
        Collection<AnnualPlanListItemVO> records = planLibList.getRecords();
        Map<Long, AnnualPlanListItemVO> itemVOMap = records.stream()
                .collect(Collectors.toMap(AnnualPlanListItemVO::getProjectId, v -> v));
        AnnualPlanListItemVO itemVO = itemVOMap.get(projectId);
        // 年度计划库中不包含当前项目
        if (Objects.isNull(itemVO)){
            processDetailVO.setStepStatus(StepStatusEnum.NOT_START);
            processDetailVO.setProcessName(CommonConstant.PROJECT_APPROVAL);
            processSchedule.add(processDetailVO);
            return;
        }
        Integer projectStatus = itemVO.getStatus();
        List<Integer> fieldList = CollUtils.fieldList(ANNUAL_PLAN_LIST_STATUS, ProjectStatusEnum::getCode);

        // 年度计划库中包含当前项目，但当前项目状态在待立项批复之前
       if (!fieldList.contains(projectStatus)) {
            processDetailVO.setStepStatus(StepStatusEnum.NOT_START);
            processDetailVO.setProcessName(CommonConstant.ANNUAL_PLAN);
            processSchedule.add(processDetailVO);
            return;
        }
        // 年度计划库包含当前项目且当前项目状态为待立项批复
        Integer status = itemVO.getStatus();
        if (ProjectStatusEnum.TO_BE_APPROVED.getCode().equals(status)){
            // 根据建设方案评审通过的时间获取
            NdProjectStatusChange projectStatusChange = projectStatusChangeService.getOne(Wrappers.lambdaQuery(NdProjectStatusChange.class)
                    .eq(NdProjectStatusChange::getProjectId, projectId)
                    .eq(NdProjectStatusChange::getEvent, ProjectStatusChangeEvent.PLAN_REVIEW_PASS.name())
                    .last("limit 1"));
            processDetailVO.setFinishTime(projectStatusChange.getCreateOn());
            processDetailVO.setStepStatus(StepStatusEnum.ON_GOING);

        } else {
            // 年度计划库包含当前项目且当前项目状态为待立项批复之后的状态
            // 根据立项批复的时间获取
            Project project = projectService.getById(projectId);
            LocalDate approvalDate = project.getApprovalDate();
            LocalDateTime approvalDateTime = approvalDate.atStartOfDay();
            processDetailVO.setFinishTime(approvalDateTime);
            processDetailVO.setStepStatus(StepStatusEnum.COMPLETED);
        }
        processDetailVO.setProcessName(CommonConstant.PROJECT_APPROVAL);
        processSchedule.add(processDetailVO);
    }
}
